/*	Beta 1
	Documentation:

	This script is designed to calculate the values of the LCDTimingX registers for displays from a set of variables the values of which the user specifies.

	Information about some variables:
	pXXX is the position of the value of variable XXX in the array "RegisterData".
	bYYY specifies whether the value of YYY is known. These variables (bYYY) are set by the function "WriteBooleanVariables".
	The value of a variable in the array "RegisterData" is -1 or undefined if it is not known.
	The variables in the array "Register" are ordered like this: HTP, HSP, HSW, HDP, VTR, VSP, VSW, VDP, PPL, LPP, HBP, HFP, VBP, VFP, PIXELCLOCK, IHS, IPC, IVS.

	The function "FillWithPresetValues" simply fills the input boxes with the chosen preset.
	"CalculateMissingValues" first loads the given values into the array "RegisterData" via the function "CollectValues". 
	It then calls "WriteBooleanVariables" to set the bYYY (where YYY is the register) variables which show whether a value is known.
	The script will then try to assess the missing values (where bYYY is not true). This happens in an special order.
	After this, the script checks whether all values could be assessed and outputs either an error or the values.	
	
	The function "ValidateValues" checks if some requirements to relations between the values are fulfilled.
*/

var RegisterData = new Array(18);
var bHDP, bVDP, bHTP, bHSP, bHSW, bHBP, bHFP, bPPL, bVTR, bVSP, bVSW, bVBP, bVFP, bLPP, bPIXELCLOCK, bIHS, bIPC, bIVS;
var LCDTiming0, LCDTiming1, LCDTiming2, LCDTiming3 = 0, LCDTiming4 = (0x4<<1)  | (1<<5) | (1<<11) | (1<<16), PCD, PCD_HI, PCD_LO, CPL;
const pHTP = 0, pHSP = 1, pHSW = 2, pHBP = 10, pHDP = 3, pHFP = 11, pVTR = 4, pVSP = 5, pVSW = 6, pVBP = 12, pVDP = 7, pVFP = 13, pPPL = 8, pLPP = 9, pPIXELCLOCK = 14, pIHS = 15, pIPC = 16, pIVS = 17;
const F_CLK = 100; // Clock frequency in MHz

function FillWithPresetValues(){
  var LCDType = document.Values.LCDTyp.options[document.Values.LCDTyp.selectedIndex].value; 
  switch(LCDType) {
    case "-1": return; break;
    case "1": with (document.Values){
      HTP.value = "408"; HSP.value = "353"; HSW.value = "30"; HDP.value = "320"; VTR.value = "263"; VSP.value = "244"; VSW.value = "3"; VDP.value = "240"; IHS.value="1"; LCDREG0.value = "19211e4c"; LCDREG1.value = "10040cef"; LCDREG2.value = "013f380d"; PIXELCLOCK.value = "7.5"; break;
    }
    case "2": with (document.Values){
      HTP.value = "525"; HSP.value = "495"; HSW.value = "20"; HDP.value = "480"; VTR.value = "286"; VSP.value = "274"; VSW.value = "10"; VDP.value = "272"; IHS.value="1"; LCDREG0.value = "0a0f1474"; LCDREG1.value = "0202290f"; LCDREG2.value = "01df380b"; PIXELCLOCK.value = "9.1"; break;
    }
    case "3": with (document.Values){
      HTP.value = "800"; HSP.value = "669"; HSW.value = "30"; HDP.value = "640"; VTR.value = "525"; VSP.value = "490"; VSW.value = "2"; VDP.value = "480"; IHS.value="1"; LCDREG0.value = "651d1e9c"; LCDREG1.value = "210a09df";  LCDREG2.value = "027f3804"; PIXELCLOCK.value = "25"; break;
    }
    case "4": with (document.Values){
      HTP.value = "1056"; HSP.value = "853"; HSW.value = "128"; HDP.value = "800"; VTR.value = "525"; VSP.value = "490"; VSW.value = "2"; VDP.value = "480"; IHS.value="0"; LCDREG0.value = "4b3580c4"; LCDREG1.value = "210a09df";  LCDREG2.value = "031f2802"; PIXELCLOCK.value = "66"; break;
    }
  }
  with (document.Values){
     PPL.value = ((Number(HDP.value) / 16) - 1);
     LPP.value = (Number(VDP.value) - 1) 
     HBP.value = Number(HTP.value) - (Number(HSP.value) + Number(HSW.value)); 
     HFP.value = Number(HTP.value) - (Number(HDP.value) + Number(HSW.value) + Number(HBP.value));
     VBP.value = Number(VTR.value) - (Number(VSP.value) + Number(VSW.value));
     VFP.value = Number(VTR.value) - (Number(VDP.value) + Number(VSW.value) + Number(VBP.value));
      "20"; //wtf who put this here 
     IPC.value = "1";
     IVS.value = "1";
     LCDREG3.value = PadWithZeros(LCDTiming3.toString(16),8);
     LCDREG4.value = PadWithZeros(LCDTiming4.toString(16),8);
     args.value = "video=tmpa9xxfb:"+LCDREG0.value+":"+LCDREG1.value+":"+LCDREG2.value;
    }
  document.getElementById("status").setAttribute("class", "NOTE");
  document.getElementById("status").value = "Calculation successful!";
}

function assessHValue(pos){
  WriteBooleanVariables(); // Update
  RegisterData[pos] = -1;
  switch(pos){
  case pHTP: // HTP
    if (bHSP && bHSW && bHBP) RegisterData[pHTP] = RegisterData[pHSP] + RegisterData[pHSW] + RegisterData[pHBP];    
    return RegisterData[pHTP];
  break;
  case pHSP: // HSP
    if (bHDP && bHFP) RegisterData[pHSP] = RegisterData[pHDP] + RegisterData[pHFP]; 
      else if (bHSW && bHBP && bHTP) RegisterData[pHSP] = RegisterData[pHTP] - (RegisterData[pHSW] + RegisterData[pHBP]);
    return RegisterData[pHSP];
  break;
  case pHSW: // HSW
    if (bHTP && bHSP  && bHBP) RegisterData[pHSW] = RegisterData[pHTP] - (RegisterData[pHSP] + RegisterData[pHBP]); 
       else if (bHDP && bHFP && bHTP && bHBP) RegisterData[pHSW] = RegisterData[pHTP] - (RegisterData[pHDP] + RegisterData[pHFP] + RegisterData[pHBP]);
    return RegisterData[pHSW]; 
  break;
  case pHDP: // HDP
    if (bHFP && bHSP) RegisterData[pHDP] = RegisterData[pHSP] - RegisterData[pHFP];  
      else if (bHSW && bHBP && bHTP) RegisterData[pHDP] = assessHValue(pHSP) - RegisterData[pHFP];
        else if (bPPL) RegisterData[pHDP] = ((RegisterData[pPPL] + 1) * 16); 
    return RegisterData[pHDP];
   break;
   case pHBP: // HBP
     if (bHSW && bHSP && bHTP) RegisterData[pHBP] = RegisterData[pHTP] - (RegisterData[pHSP] + RegisterData[pHSW]);
        else if (bHSW && bHDP && bHFP && bHTP) RegisterData[pHBP] = RegisterData[pHTP] - (RegisterData[pHDP] + RegisterData[pHFP] + RegisterData[pHSW]);
     return RegisterData[pHBP];
   break;
   case pHFP: // HFP     
     if (bHDP &&  bHSP) RegisterData[pHFP] = RegisterData[pHSP] - RegisterData[pHDP];
       else if (bHDP && bHSW && bHBP && bHTP) RegisterData[pHFP] = assessHValue(pHSP) - RegisterData[pHDP];    
         else if (bPPL && bHSP) RegisterData[pHFP] = ((RegisterData[pPPL] + 1) * 16) - RegisterData[pHDP];  
     return RegisterData[pHFP];
   break;
  }
  return RegisterData[pos];
}

function assessVValue(pos){ 
  WriteBooleanVariables(); // Update
  RegisterData[pos] = -1;
  switch(pos){
  case pVTR: // VTR
    if (bVSP && bVSW && bVBP) RegisterData[pVTR] = RegisterData[pVSP] + RegisterData[pVSW] + RegisterData[pVBP];    
    return RegisterData[pVTR];
  break;
  case pVSP: // VSP
    if (bVDP && bVFP) RegisterData[pVSP] = RegisterData[pVDP] + RegisterData[pVFP]; 
      else if (bVSW && bVBP && bVTR) RegisterData[pVSP] = RegisterData[pVTR] - (RegisterData[pVSW] + RegisterData[pVBP]);
    return RegisterData[pVSP];
  break;
  case pVSW: // VSW
    if (bVTR && bVSP  && bVBP) RegisterData[pVSW] = RegisterData[pVTR] - (RegisterData[pVSP] + RegisterData[pVBP]); 
       else if (bVDP && bVFP && bVTR && bVBP) RegisterData[pVSW] = RegisterData[pVTR] - (RegisterData[pVDP] + RegisterData[pVFP] + RegisterData[pVBP]);
    return RegisterData[pVSW];
  break;
  case pVDP: // VDP
    if (bVFP && bVSP) RegisterData[pVDP] = RegisterData[pVSP] - RegisterData[pVFP];  
      else if (bVSW && bVBP && bVTR) RegisterData[pVDP] = assessVValue(pVSP) - RegisterData[pVFP];
         else if (bLPP) RegisterData[pVDP] = (RegisterData[pLPP] + 1);
    return RegisterData[pVDP];
   break;
   case pVBP: // VBP
     if (bVSW && bVSP && bVTR) RegisterData[pVBP] = RegisterData[pVTR] - (RegisterData[pVSP] + RegisterData[pVSW]);
        else if (bVSW && bVDP && bVFP && bVTR) RegisterData[pVBP] = RegisterData[pVTR] - (RegisterData[pVDP] + RegisterData[pVFP] + RegisterData[pVSW]);
     return RegisterData[pVBP];
   break;
   case pVFP: // VFP    
     if (bVDP &&  bVSP) RegisterData[pVFP] = RegisterData[pVSP] - RegisterData[pVDP];
       else if (bVDP && bVSW && bVBP && bVTR) RegisterData[pVFP] = assessVValue(pVSP) - RegisterData[pVDP]; 
         else if (bLPP && bVSP) RegisterData[pVFP] = (RegisterData[pLPP] + 1) - RegisterData[pVDP];  
     return RegisterData[pVFP];
   break; 
  }
 return RegisterData[pos];
}

function assessSpecialValue(pos){ 
  WriteBooleanVariables(); // Update
  RegisterData[pos] = -1;
  switch(pos){
    case pPPL:
      if (bHDP) RegisterData[pPPL] = ((RegisterData[pHDP] / 16) - 1);
    break;
    case pLPP:
      if (bVDP) RegisterData[pLPP] = (RegisterData[pVDP] - 1);
    break;
  }
  return RegisterData[pos];
}

function CalculateMissingValues(){ 
  var i;
  CollectValues(); // Collect the values the user entered
  WriteBooleanVariables(); // Write the variables which show whether a value is known / has been entered by the user
  // The values are not being assessed directly if they are unknown, because the value being assessed might need to know values which are generally known, but not yet at that point. 
  // Horizontal
  if (!bHDP) assessHValue(pHDP);
  if (!bHFP) assessHValue(pHFP);
  if (!bHSP) assessHValue(pHSP);
  if (!bHSW) assessHValue(pHSW);
  if (!bHBP) assessHValue(pHBP);
  if (!bHTP) assessHValue(pHTP);
  // Vertical
  if (!bVDP) assessVValue(pVDP);
  if (!bVFP) assessVValue(pVFP);
  if (!bVSP) assessVValue(pVSP);
  if (!bVSW) assessVValue(pVSW);
  if (!bVBP) assessVValue(pVBP);
  if (!bVTR) assessVValue(pVTR);
  // Special
  if (!bPPL) assessSpecialValue(pPPL);
  if (!bLPP) assessSpecialValue(pLPP);
  // Check if lower 14 values could be assessed - if not, there is some input missing (probably)
  // The lower 14 values are enough to calculate everything except for LCDTiming2, so we would continue even without the upper 4 values, which are checked in "WriteAllValues".
  for (i = 0; i <= 13; i++){ 
    if (RegisterData[i] == -1) {
      document.getElementById("status").value = "Not enough input!"
      return -1;
    }
  }
  document.getElementById("status").setAttribute("class", "NOTE");
  document.getElementById("status").value = "Calculation successful!";
  WriteAllValues();
  return 1;
}

function GetLCDTiming2(){
  if (bPIXELCLOCK && bIVS && bIHS && bIPC) { // Values needed for calculation of LCDTiming2
    // CPL: #define CPL                       ((XSIZE_PHYS-1) & 0x3FF) // cycles per line
    // XSIZE_PHYS = HDP
    // HDP is known at this point
    CPL = ((RegisterData[pHDP] - 1) & 0x3FF);
    PCD = Math.round(F_CLK / RegisterData[pPIXELCLOCK]);
    PCD_HI = (PCD & 0x3E0); // Upper 5 Bits of PCD, 0x3E0 = 0b1111100000
    PCD_LO = (PCD & 0x1F); // Lower 5 Bits of PCD, 0x1F = 0b11111 
    LCDTiming2 = ((PCD_HI << 27) | (CPL << 16) | (RegisterData[pIPC] << 13) | (RegisterData[pIHS] << 12) | (RegisterData[pIVS] << 11) | (PCD_LO << 0));
    return 1;
  }
  else {
    LCDTiming2 = "N/A";
    return -1;
  }
}

function WriteAllValues(){
  with (document.Values){
    HTP.value = RegisterData[pHTP];
    HSP.value = RegisterData[pHSP];
    HSW.value = RegisterData[pHSW];
    HDP.value = RegisterData[pHDP];
    VTR.value = RegisterData[pVTR];
    VSP.value = RegisterData[pVSP];
    VSW.value = RegisterData[pVSW];
    VDP.value = RegisterData[pVDP];
    PPL.value = RegisterData[pPPL];
    LPP.value = RegisterData[pLPP];
    HBP.value = RegisterData[pHBP];
    HFP.value = RegisterData[pHFP];
    VBP.value = RegisterData[pVBP];
    VFP.value = RegisterData[pVFP];
    LCDTiming0 = ((RegisterData[pPPL] << 2) | (RegisterData[pHSW] << 8 ) | (RegisterData[pHFP] << 16) | (RegisterData[pHBP] << 24 ));
    LCDTiming1 = ((RegisterData[pVBP] << 24)| (RegisterData[pVFP] << 16) | (RegisterData[pVSW] << 10) | (RegisterData[pLPP]));
    if (GetLCDTiming2() == -1) document.getElementById("status").value = "Calculation successful! However, all last four fields are needed for the calculation of  LCDReg[2].";
    LCDREG0.value = PadWithZeros(LCDTiming0.toString(16),8);  // Convert to hexadecimal format
    LCDREG1.value = PadWithZeros(LCDTiming1.toString(16),8);
    LCDREG2.value = PadWithZeros(LCDTiming2.toString(16),8);
    LCDREG3.value = PadWithZeros(LCDTiming3.toString(16),8);
    LCDREG4.value = PadWithZeros(LCDTiming4.toString(16),8);
    args.value = "video=tmpa9xxfb:"+LCDREG0.value+":"+LCDREG1.value+":"+LCDREG2.value;
  }  
}

function ValidateValues(){
  var ErrCount = 0;
  function fail(error){
    if (document.getElementById("status").value == "") document.getElementById("status").value = "Malformed values: \n"+error;
      else document.getElementById("status").value += "\n"+error;
    ErrCount++;
  }
  if (CalculateMissingValues() == -1) return; // No need to fail more
  document.getElementById("status").value = ""; // Clear messages
  document.getElementById("status").setAttribute("class", "ERROR"); // Set error class 
  CollectValues(); 
  // In the following code some mathematical relations between the register values are checked, they should always apply 
  if ((RegisterData[pHSP] + RegisterData[pHSW] + RegisterData[pHBP]) != RegisterData[pHTP]) fail("(HSP + HSW + HBP) doesn't equal HTP!");
  if ((RegisterData[pHDP] + RegisterData[pHFP]) != RegisterData[pHSP]) fail("(HDP + HFP) doesn't equal HSP!");
  if ((RegisterData[pVSP] + RegisterData[pVSW] + RegisterData[pVBP]) != RegisterData[pVTR]) fail("(VSP + VSW + VBP) doesn't equal VTR!");
  if ((RegisterData[pVDP] + RegisterData[pVFP]) != RegisterData[pVSP]) fail("(VDP + VFP) doesn't equal VSP!");
  if (RegisterData[pPPL] != ((RegisterData[pHDP] / 16) - 1)) fail("((HDP/16) - 1) doesn't equal PPL!");
  if (RegisterData[pLPP] != (RegisterData[pVDP] - 1)) fail("(VDP - 1) doesn't equal LPP!");
  if (ErrCount == 0) { 
     document.getElementById("status").setAttribute("class", "NOTE"); // Set note class
     document.getElementById("status").value = "Validation successful!";
  }
}

function CollectValues(){ // Loads the values of the user input into the array "Register" 
  Register = []; // Clear Array
  with (document.Values){
    // Horizontal
    RegisterData[pHDP] = (HDP.value == "") ? -1 : Number(HDP.value);    
    RegisterData[pHFP] = (HFP.value == "") ? -1 : Number(HFP.value);
    RegisterData[pHSP] = (HSP.value == "") ? -1 : Number(HSP.value);
    RegisterData[pHSW] = (HSW.value == "") ? -1 : Number(HSW.value);
    RegisterData[pHBP] = (HBP.value == "") ? -1 : Number(HBP.value);
    RegisterData[pHTP] = (HTP.value == "") ? -1 : Number(HTP.value); 
    // Vertical
    RegisterData[pVDP] = (VDP.value == "") ? -1 : Number(VDP.value);
    RegisterData[pVFP] = (VFP.value == "") ? -1 : Number(VFP.value);
    RegisterData[pVSP] = (VSP.value == "") ? -1 : Number(VSP.value);
    RegisterData[pVSW] = (VSW.value == "") ? -1 : Number(VSW.value);
    RegisterData[pVBP] = (VBP.value == "") ? -1 : Number(VBP.value);
    RegisterData[pVTR] = (VTR.value == "") ? -1 : Number(VTR.value);   
    // Special
    RegisterData[pPPL] = (PPL.value == "") ? -1 : Number(PPL.value);
    RegisterData[pLPP] = (LPP.value == "") ? -1 : Number(LPP.value);   
    RegisterData[pPIXELCLOCK] = (PIXELCLOCK.value == "") ? -1 : Number(PIXELCLOCK.value);
    RegisterData[pIPC] = (IPC.value == "") ? -1 : Number(IPC.value);
    RegisterData[pIHS] = (IHS.value == "") ? -1 : Number(IHS.value);
    RegisterData[pIVS] = (IVS.value == "") ? -1 : Number(IVS.value);
  }
}

function WriteBooleanVariables(){ // Checks for unknown values and writes to bYYY variables
  bVTR 		= (RegisterData[pVTR] != -1) ? true : false;
  bVSP 		= (RegisterData[pVSP] != -1) ? true : false;
  bVSW 		= (RegisterData[pVSW] != -1) ? true : false;
  bVBP 		= (RegisterData[pVBP] != -1) ? true : false;
  bVDP 		= (RegisterData[pVDP] != -1) ? true : false;
  bVFP 		= (RegisterData[pVFP] != -1) ? true : false;
  bLPP 		= (RegisterData[pLPP] != -1) ? true : false;
  bHTP 		= (RegisterData[pHTP] != -1) ? true : false;
  bHSP 		= (RegisterData[pHSP] != -1) ? true : false;
  bHSW 		= (RegisterData[pHSW] != -1) ? true : false;
  bHDP 		= (RegisterData[pHDP] != -1) ? true : false;
  bHBP 		= (RegisterData[pHBP] != -1) ? true : false;
  bHFP 		= (RegisterData[pHFP] != -1) ? true : false;
  bPPL 		= (RegisterData[pPPL] != -1) ? true : false;
  bPIXELCLOCK 	= (RegisterData[pPIXELCLOCK] != -1) ? true : false;
  bIPC 		= (RegisterData[pIPC] != -1) ? true : false;
  bIHS 		= (RegisterData[pIHS] != -1) ? true : false;
  bIVS		= (RegisterData[pIVS] != -1) ? true : false;
}

function RemoveNonDigits(element){
  element.value = element.value.replace(/[^\d]/g,""); // Remove all non digits via RegEx
}

function PadWithZeros(num,count)
{
var numZeropad = num + '';
while(numZeropad.length < count) {
numZeropad = "0" + numZeropad;
}
return numZeropad;
}

